# pylint: disable=no-self-use
"""
The main entry point for ResInsight connections
The Instance class contained have static methods launch and find for
creating connections to ResInsight
"""

import os
import socket
import logging
import time
import tempfile
import signal

import grpc

import App_pb2
import App_pb2_grpc
import Commands_pb2
import Commands_pb2_grpc
from Definitions_pb2 import Empty

import RiaVersionInfo

from .project import Project
from .retry_policy import ExponentialBackoffRetryPolicy
from .grpc_retry_interceptor import RetryOnRpcErrorClientInterceptor
from .generated.generated_classes import CommandRouter


class Instance:
    """The ResInsight Instance class. Use to launch or find existing ResInsight instances

    Attributes:
        launched (bool): Tells us whether the application was launched as a new process.
            If the application was launched we may need to close it when exiting the script.
        commands (Commands): Command executor. Set when creating an instance.
        project (Project): Current project in ResInsight.
            Set when creating an instance and updated when opening/closing projects.
    """

    @staticmethod
    def __is_port_in_use(port):
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as my_socket:
            my_socket.settimeout(0.2)
            return my_socket.connect_ex(("localhost", port)) == 0

    @staticmethod
    def __is_valid_port(port):
        location = "localhost:" + str(port)
        channel = grpc.insecure_channel(
            location, options=[("grpc.enable_http_proxy", False)]
        )
        app = App_pb2_grpc.AppStub(channel)
        try:
            app.GetVersion(Empty(), timeout=1)
        except grpc.RpcError:
            return False
        return True

    @staticmethod
    def __read_port_number_from_file(file_path):
        retry_count = 0
        while not os.path.exists(file_path) and retry_count < 60:
            time.sleep(1)
            retry_count = retry_count + 1

        print("Portnumber file retry count : ", retry_count)

        if os.path.isfile(file_path):
            with open(file_path) as f:
                value = f.readline()
                return int(value)
        else:
            return -1

    @staticmethod
    def __kill_process(pid):
        """
        Kill the process with a given pid.
        """
        if hasattr(signal, "CTRL_C_EVENT"):
            # windows does not have kill
            os.kill(pid, signal.CTRL_C_EVENT)
        else:
            # linux/unix
            os.kill(pid, signal.SIGKILL)

    @staticmethod
    def launch(
        resinsight_executable="",
        console=False,
        launch_port=-1,
        command_line_parameters=None,
    ):
        """Launch a new Instance of ResInsight. This requires the environment variable
        RESINSIGHT_EXECUTABLE to be set or the parameter resinsight_executable to be provided.
        The RESINSIGHT_GRPC_PORT environment variable can be set to an alternative port number.

        Args:
            resinsight_executable (str): Path to a valid ResInsight executable. If set
                will take precedence over what is provided in the RESINSIGHT_EXECUTABLE
                environment variable.
            console (bool): If True, launch as console application, without GUI.
            launch_port(int): If -1 will use the default port 50051 or RESINSIGHT_GRPC_PORT
                             if anything else, ResInsight will try to launch with this port.
                             If 0 a random port will be used.
            command_line_parameters(list): Additional parameters as string entries in the list.
        Returns:
            Instance: an instance object if it worked. None if not.
        """

        requested_port = 50051
        port_env = os.environ.get("RESINSIGHT_GRPC_PORT")
        if port_env:
            requested_port = int(port_env)
        if launch_port != -1:
            requested_port = launch_port

        if not resinsight_executable:
            resinsight_executable = os.environ.get("RESINSIGHT_EXECUTABLE")
            if not resinsight_executable:
                print(
                    "ERROR: Could not launch ResInsight because the environment variable"
                    " RESINSIGHT_EXECUTABLE is not set"
                )
                return None

        print("Trying to launch", resinsight_executable)

        if command_line_parameters is None:
            command_line_parameters = []
        elif isinstance(command_line_parameters, str):
            command_line_parameters = [str]

        with tempfile.TemporaryDirectory() as tmp_dir_path:
            port_number_file = tmp_dir_path + "/portnumber.txt"
            parameters = [
                "ResInsight",
                "--server",
                requested_port,
                "--portnumberfile",
                port_number_file,
            ] + command_line_parameters
            if console:
                print("Launching as console app")
                parameters.append("--console")

            # Stringify all parameters
            for i in range(0, len(parameters)):
                parameters[i] = str(parameters[i])

            pid = os.spawnv(os.P_NOWAIT, resinsight_executable, parameters)
            if pid:
                port = Instance.__read_port_number_from_file(port_number_file)
                if port == -1:
                    print("Unable to read port number. Launch failed.")
                    # Need to kill the process using PID since there is no  GRPC connection to use.
                    Instance.__kill_process(pid)
                else:
                    instance = Instance(port=port, launched=True)
                    return instance
        return None

    @staticmethod
    def find(start_port=50051, end_port=50071):
        """Search for an existing Instance of ResInsight by testing ports.

        By default we search from port 50051 to 50071 or if the environment
        variable RESINSIGHT_GRPC_PORT is set we search
        RESINSIGHT_GRPC_PORT to RESINSIGHT_GRPC_PORT+20

        Args:
            start_port (int): start searching from this port
            end_port (int): search up to but not including this port
        """
        port_env = os.environ.get("RESINSIGHT_GRPC_PORT")
        if port_env:
            print("Got port " + port_env + " from environment")
            start_port = int(port_env)
            end_port = start_port + 20

        for try_port in range(start_port, end_port):
            print("Trying port " + str(try_port))
            if Instance.__is_port_in_use(try_port) and Instance.__is_valid_port(
                try_port
            ):
                return Instance(port=try_port)

        print(
            "Error: Could not find any ResInsight instances responding between ports "
            + str(start_port)
            + " and "
            + str(end_port)
        )
        return None

    def __execute_command(self, **command_params):
        return self.commands.Execute(Commands_pb2.CommandParams(**command_params))

    def __check_version(self):
        try:
            major_version_ok = self.major_version() == int(
                RiaVersionInfo.RESINSIGHT_MAJOR_VERSION
            )
            minor_version_ok = self.minor_version() == int(
                RiaVersionInfo.RESINSIGHT_MINOR_VERSION
            )
            return True, major_version_ok and minor_version_ok
        except grpc.RpcError:
            return False, False

    def __init__(self, port=50051, launched=False):
        """Attempts to connect to ResInsight at aa specific port on localhost

        Args:
            port(int): port number
        """
        logging.basicConfig()
        self.location = "localhost:" + str(port)

        self.channel = grpc.insecure_channel(
            self.location, options=[("grpc.enable_http_proxy", False)]
        )
        self.launched = launched
        self.commands = Commands_pb2_grpc.CommandsStub(self.channel)

        # Main version check package
        self.app = App_pb2_grpc.AppStub(self.channel)

        self._check_connection_and_version(self.channel, launched, self.location)

        # Intercept UNAVAILABLE errors and retry on failures
        interceptors = (
            RetryOnRpcErrorClientInterceptor(
                retry_policy=ExponentialBackoffRetryPolicy(
                    min_backoff=100, max_backoff=5000, max_num_retries=20
                ),
                status_for_retry=(grpc.StatusCode.UNAVAILABLE,),
            ),
        )

        intercepted_channel = grpc.intercept_channel(self.channel, *interceptors)

        # Recreate command stubs with the retry policy
        self.commands = Commands_pb2_grpc.CommandsStub(intercepted_channel)

        # Service packages
        self.project = Project.create(intercepted_channel)

        # Command Router object used as entry point for independent processing functions
        self.command_router = CommandRouter(
            self.app.GetPdmObject(Empty()), intercepted_channel
        )

        path = os.getcwd()
        self.set_start_dir(path=path)

    def _check_connection_and_version(self, channel, launched, location):
        connection_ok = False
        version_ok = False

        retry_policy = ExponentialBackoffRetryPolicy()
        if self.launched:
            for num_tries in range(0, retry_policy.num_retries()):
                connection_ok, version_ok = self.__check_version()
                if connection_ok:
                    break
                retry_policy.sleep(num_tries)
        else:
            connection_ok, version_ok = self.__check_version()

        if not connection_ok:
            if self.launched:
                raise Exception(
                    "Error: Could not connect to resinsight at ",
                    location,
                    ".",
                    retry_policy.time_out_message(),
                )
            raise Exception("Error: Could not connect to resinsight at ", location)
        if not version_ok:
            raise Exception(
                "Error: Wrong Version of ResInsight at ",
                location,
                self.version_string(),
                " ",
                self.client_version_string(),
            )

    def __version_message(self):
        return self.app.GetVersion(Empty())

    def set_start_dir(self, path):
        """Set current start directory

        Arguments:
            path (str): path to directory

        """
        return self.__execute_command(
            setStartDir=Commands_pb2.FilePathRequest(path=path)
        )

    def set_export_folder(self, export_type, path, create_folder=False):
        """
        Set the export folder used for all export functions

        **Parameters**::

            Parameter        | Description                                  | Type
            ---------------- | -------------------------------------------- | -----
            export_type      | String specifying what to export             | String
            path             | Path to folder                               | String
            create_folder    | Create folder if it doesn't exist?           | Boolean

        **Enum export_type**::

            Option          | Description
            --------------- | ------------
            "COMPLETIONS"   |
            "SNAPSHOTS"     |
            "PROPERTIES"    |
            "STATISTICS"    |

        """
        return self.__execute_command(
            setExportFolder=Commands_pb2.SetExportFolderRequest(
                type=export_type, path=path, createFolder=create_folder
            )
        )

    def set_main_window_size(self, width, height):
        """
        Set the main window size in pixels

        **Parameters**::

            Parameter | Description      | Type
            --------- | ---------------- | -----
            width     | Width in pixels  | Integer
            height    | Height in pixels | Integer

        """
        return self.__execute_command(
            setMainWindowSize=Commands_pb2.SetWindowSizeParams(
                width=width, height=height
            )
        )

    def set_plot_window_size(self, width, height):
        """
        Set the plot window size in pixels

        **Parameters**::

            Parameter | Description      | Type
            --------- | ---------------- | -----
            width     | Width in pixels  | Integer
            height    | Height in pixels | Integer
        """
        return self.__execute_command(
            setPlotWindowSize=Commands_pb2.SetWindowSizeParams(
                width=width, height=height
            )
        )

    def major_version(self):
        """Get an integer with the major version number"""
        return self.__version_message().major_version

    def minor_version(self):
        """Get an integer with the minor version number"""
        return self.__version_message().minor_version

    def patch_version(self):
        """Get an integer with the patch version number"""
        return self.__version_message().patch_version

    def version_string(self):
        """Get a full version string, i.e. 2019.04.01"""
        return (
            str(self.major_version())
            + "."
            + str(self.minor_version())
            + "."
            + str(self.patch_version())
        )

    def client_version_string(self):
        """Get a full version string, i.e. 2019.04.01"""
        version_string = RiaVersionInfo.RESINSIGHT_MAJOR_VERSION + "."
        version_string += RiaVersionInfo.RESINSIGHT_MINOR_VERSION + "."
        version_string += RiaVersionInfo.RESINSIGHT_PATCH_VERSION
        return version_string

    def exit(self):
        """Tell ResInsight instance to quit"""
        print("Telling ResInsight to Exit")
        return self.app.Exit(Empty())

    def is_console(self):
        """Returns true if the connected ResInsight instance is a console app"""
        return self.app.GetRuntimeInfo(
            Empty()
        ).app_type == App_pb2.ApplicationTypeEnum.Value("CONSOLE_APPLICATION")

    def is_gui(self):
        """Returns true if the connected ResInsight instance is a GUI app"""
        return self.app.GetRuntimeInfo(
            Empty()
        ).app_type == App_pb2.ApplicationTypeEnum.Value("GUI_APPLICATION")



"""
ResInsight Well Log Plot plot module
"""

import Commands_pb2

from .plot import Plot
from .pdmobject import PdmObjectBase, add_method
from .resinsight_classes import WellLogPlot


@add_method(WellLogPlot)
def export_data_as_las(
    self,
    export_folder,
    file_prefix="",
    export_tvdrkb=False,
    capitalize_file_names=False,
    resample_interval=0.0,
    convert_to_standard_units=False,
):
    """Export LAS file(s) for the current plot

    Arguments:
        export_folder(str): The path to export to. By default will use the global export folder
        file_prefix (str): Exported file name prefix
        export_tvdrkb(bool): Export in TVD-RKB format
        capitalize_file_names(bool): Make all file names upper case
        resample_interval(double): if > 0.0 the files will be resampled

    Returns:
        A list of files exported
    """
    res = self._execute_command(
        exportWellLogPlotData=Commands_pb2.ExportWellLogPlotDataRequest(
            exportFormat="LAS",
            viewId=self.id,
            exportFolder=export_folder,
            filePrefix=file_prefix,
            exportTvdRkb=export_tvdrkb,
            capitalizeFileNames=capitalize_file_names,
            resampleInterval=resample_interval,
            convertCurveUnits=convert_to_standard_units,
        )
    )
    return res.exportWellLogPlotDataResult.exportedFiles


@add_method(WellLogPlot)
def export_data_as_ascii(
    self, export_folder, file_prefix="", capitalize_file_names=False
):
    """Export LAS file(s) for the current plot

    Arguments:
        export_folder(str): The path to export to. By default will use the global export folder
        file_prefix (str): Exported file name prefix
        capitalize_file_names(bool): Make all file names upper case

    Returns:
        A list of files exported
    """
    res = self._execute_command(
        exportWellLogPlotData=Commands_pb2.ExportWellLogPlotDataRequest(
            exportFormat="ASCII",
            viewId=self.id,
            exportFolder=export_folder,
            filePrefix=file_prefix,
            exportTvdRkb=False,
            capitalizeFileNames=capitalize_file_names,
            resampleInterval=0.0,
        )
    )
    return res.exportWellLogPlotDataResult.exportedFiles



# Load ResInsight Processing Server Client Library
import rips
import tempfile
from os.path import expanduser
from pathlib import Path

# Connect to ResInsight instance
resinsight = rips.Instance.find()


home_dir = expanduser("~")

export_folder = tempfile.mkdtemp()

directory_path = "resprojects/webviz-subsurface-testdata/reek_history_match/"
# directory_path = "e:/gitroot/webviz-subsurface-testdata/reek_history_match"


case_file_paths = []
num_realizations = 9
num_iterations = 4


for realization in range(0, num_realizations):
    for iteration in range(0, num_iterations):
        realization_dir = "realization-" + str(realization)
        iteration_dir = "iter-" + str(iteration)
        egrid_name = "eclipse/model/5_R001_REEK-" + str(realization) + ".EGRID"
        path = Path(
            home_dir, directory_path, realization_dir, iteration_dir, egrid_name
        )
        case_file_paths.append(path)

k_indexes = [4, 10]

command_router = resinsight.command_router

for path in case_file_paths:
    path_name = path.as_posix()

    command_router.extract_surfaces(path_name, k_indexes)



###################################################################################
# This example will connect to ResInsight, retrieve a list of cases and print info
#
###################################################################################

# Import the ResInsight Processing Server Module
import rips

# Connect to ResInsight
resinsight = rips.Instance.find()
if resinsight is not None:
    # Get a list of all cases
    cases = resinsight.project.cases()

    print("Got " + str(len(cases)) + " cases: ")
    for case in cases:
        print("Case id: " + str(case.id))
        print("Case name: " + case.name)
        print("Case type: " + case.__class__.__name__)
        print("Case file name: " + case.file_path)
        print("Case reservoir bounding box:", case.reservoir_boundingbox())

        timesteps = case.time_steps()
        for t in timesteps:
            print("Year: " + str(t.year))
            print("Month: " + str(t.month))

        if isinstance(case, rips.EclipseCase):
            print("Getting coarsening info for case: ", case.name, case.id)
            coarsening_info = case.coarsening_info()
            if coarsening_info:
                print("Coarsening information:")

            for c in coarsening_info:
                print(
                    "[{}, {}, {}] - [{}, {}, {}]".format(
                        c.min.x, c.min.y, c.min.z, c.max.x, c.max.y, c.max.z
                    )
                )



import sys
import os
import math
import pytest
import grpc
import tempfile
import time
import multiprocessing

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot


def launch_resinsight(sec=1):
    instance = rips.Instance.launch(console=True, launch_port=0)
    print(instance.location)

    print(f"Sleeping for {sec} second(s): ", instance.location)
    time.sleep(sec)
    print(f"Done sleeping", instance.location)

    instance.exit()


def test_launch_sequential(rips_instance, initialize_test):
    instance_list = []
    for i in range(4):
        rips_instance = rips.Instance.launch(console=True)
        instance_list.append(rips_instance)

    for instance in instance_list:
        print(instance)
        instance.exit()


def test_launch_parallell(rips_instance, initialize_test):
    process_list = []

    instance_count = 10
    for i in range(instance_count):
        process = multiprocessing.Process(target=launch_resinsight)
        process_list.append(process)

    for process in process_list:
        process.start()

    # completing process
    for p in process_list:
        p.join()



# Import the tempfile module
import tempfile

# Load ResInsight Processing Server Client Library
import rips

# Connect to ResInsight instance
resInsight = rips.Instance.find()

# Get a list of all plots
plots = resInsight.project.plots()

export_folder = tempfile.mkdtemp()

print("Exporting to: " + export_folder)

for plot in plots:
    plot.export_snapshot(export_folder=export_folder)
    plot.export_snapshot(export_folder=export_folder, output_format="PDF")
    if isinstance(plot, rips.WellLogPlot):
        plot.export_data_as_las(export_folder=export_folder)
        plot.export_data_as_ascii(export_folder=export_folder)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import Commands_pb2 as Commands__pb2


class CommandsStub(object):
    """
    The Commands service handles generic ResInsight RicfCommandObject-commands
    The CommandParams is used to deduce command name from the chosen oneof-message
    .. and the parameters are in the oneof-message itself.
    Note that we several times duplicate nearly the same message. This is because
    it is not recommended to reuse existing messages for different purposes.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Execute = channel.unary_unary(
                '/rips.Commands/Execute',
                request_serializer=Commands__pb2.CommandParams.SerializeToString,
                response_deserializer=Commands__pb2.CommandReply.FromString,
                )


class CommandsServicer(object):
    """
    The Commands service handles generic ResInsight RicfCommandObject-commands
    The CommandParams is used to deduce command name from the chosen oneof-message
    .. and the parameters are in the oneof-message itself.
    Note that we several times duplicate nearly the same message. This is because
    it is not recommended to reuse existing messages for different purposes.
    """

    def Execute(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_CommandsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Execute': grpc.unary_unary_rpc_method_handler(
                    servicer.Execute,
                    request_deserializer=Commands__pb2.CommandParams.FromString,
                    response_serializer=Commands__pb2.CommandReply.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'rips.Commands', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Commands(object):
    """
    The Commands service handles generic ResInsight RicfCommandObject-commands
    The CommandParams is used to deduce command name from the chosen oneof-message
    .. and the parameters are in the oneof-message itself.
    Note that we several times duplicate nearly the same message. This is because
    it is not recommended to reuse existing messages for different purposes.
    """

    @staticmethod
    def Execute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rips.Commands/Execute',
            Commands__pb2.CommandParams.SerializeToString,
            Commands__pb2.CommandReply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



#######################################################
#
# This file shows how to import properties for a
# grid case created with .ROFFASC files
#
# Same procedure can also be used for .GRDECL files
#
#######################################################


# Access to environment variables and path tools
import os

# Load ResInsight Processing Server Client Library
import rips

# Connect to ResInsight instance
resinsight = rips.Instance.find()

# This requires the TestModels to be installed with ResInsight (RESINSIGHT_BUNDLE_TESTMODELS):
resinsight_exe_path = os.environ.get("RESINSIGHT_EXECUTABLE")

# Get the TestModels path from the executable path
resinsight_install_path = os.path.dirname(resinsight_exe_path)
test_models_path = os.path.join(resinsight_install_path, "TestModels")

# Get the .roff case
roff_case_path = os.path.join(
    test_models_path, "reek/reek_box_grid_w_out_props.roffasc"
)
roff_case = resinsight.project.load_case(roff_case_path)

# PORO and EQLNUM should not be among available properties yet
for prop in roff_case.available_properties("INPUT_PROPERTY"):
    print(prop)

# Import properties with file paths
poro_property_path = os.path.join(
    test_models_path, "reek/reek_box_PORO_property.roffasc"
)
eqlnum_property_path = os.path.join(
    test_models_path, "reek/reek_box_EQLNUM_property.roffasc"
)
roff_case.import_properties(file_names=[poro_property_path, eqlnum_property_path])

# PORO and EQLNUM should now be among available properties
for prop in roff_case.available_properties("INPUT_PROPERTY"):
    print(prop)



###############################################################################
# This example will show setting time step, window size and export snapshots and properties
###############################################################################
import os
import tempfile
import rips

# Load instance
resinsight = rips.Instance.find()

# Set window sizes
resinsight.set_main_window_size(width=800, height=500)
resinsight.set_plot_window_size(width=1000, height=1000)


# Retrieve first case
case = resinsight.project.cases()[0]

# Get a view
view1 = case.views()[0]

# Clone the view
view2 = view1.clone()

# Set the time step for view1 only
view1.set_time_step(time_step=2)

# Set cell result to SOIL
view1.apply_cell_result(result_type="DYNAMIC_NATIVE", result_variable="SOIL")


# Create a temporary directory which will disappear at the end of this script
# If you want to keep the files, provide a good path name instead of tmpdirname
with tempfile.TemporaryDirectory(prefix="rips") as tmpdirname:
    print("Temporary folder: ", tmpdirname)

    # Set export folder for snapshots and properties
    resinsight.set_export_folder(export_type="SNAPSHOTS", path=tmpdirname)
    resinsight.set_export_folder(export_type="PROPERTIES", path=tmpdirname)

    # Export all snapshots
    resinsight.project.export_snapshots()

    assert len(os.listdir(tmpdirname)) > 0

    # Export properties in the view
    view1.export_property()

    # Check that the exported file exists
    expected_file_name = case.name + "-" + str("3D_View") + "-" + "T2" + "-SOIL"
    full_path = tmpdirname + "/" + expected_file_name

    # Print contents of temporary folder
    print(os.listdir(tmpdirname))

    assert os.path.exists(full_path)



# Load ResInsight Processing Server Client Library
import rips
import tempfile
from os.path import expanduser
from pathlib import Path

# Connect to ResInsight instance
resinsight = rips.Instance.find()


home_dir = expanduser("~")

export_folder = tempfile.mkdtemp()

directory_path = "resprojects/webviz-subsurface-testdata/reek_history_match/"
# directory_path = "e:/gitroot/webviz-subsurface-testdata/reek_history_match"


case_file_paths = []
num_realizations = 9
num_iterations = 4


for realization in range(0, num_realizations):
    for iteration in range(0, num_iterations):
        realization_dir = "realization-" + str(realization)
        iteration_dir = "iter-" + str(iteration)
        egrid_name = "eclipse/model/5_R001_REEK-" + str(realization) + ".EGRID"
        path = Path(
            home_dir, directory_path, realization_dir, iteration_dir, egrid_name
        )
        case_file_paths.append(path)

k_indexes = [4, 10]

for path in case_file_paths:
    # Load a case
    path_name = path.as_posix()
    case = resinsight.project.load_case(path_name)

    if resinsight.project.has_warnings():
        for warning in resinsight.project.warnings():
            print(warning)

    surface_collection = resinsight.project.descendants(rips.SurfaceCollection)[0]

    for k_index in k_indexes:
        print("Generating surface K layer " + str(k_index) + " for case " + path_name)

        surface = surface_collection.new_surface(case, k_index)
        print("Surface: ", surface)

        parent_path = path.parent
        export_folder_path = Path(parent_path, "surfaceexport")
        export_folder_path.mkdir(parents=True, exist_ok=True)

        export_file = Path(export_folder_path, "surf_" + str(k_index) + ".ts")
        print("Exporting to " + export_file.as_posix())
        surface.export_to_file(export_file.as_posix())

    # Close project to avoid aggregated memory usage
    # Can be replaced when case.close() is implemented
    resinsight.project.close()



import sys
import os
from typing import Any, Dict, List, TypedDict
import math

from generated.generated_classes import Case

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot


class NameAndPath(TypedDict):
    # Typed dict container for name of property and path of property file
    name: str
    path: str


def check_corner(actual, expected):
    assert math.isclose(actual.x, expected[0], abs_tol=0.1)
    assert math.isclose(actual.y, expected[1], abs_tol=0.1)
    assert math.isclose(actual.z, expected[2], abs_tol=0.1)


def test_10k(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=casePath)
    assert len(case.grids()) == 2
    grid = case.grid(index=0)
    dimensions = grid.dimensions()
    assert dimensions.i == 90
    assert dimensions.j == 96
    assert dimensions.k == 36

    cell_centers = grid.cell_centers()
    assert len(cell_centers) == (dimensions.i * dimensions.j * dimensions.k)

    property_data_index = grid.property_data_index_from_ijk(31, 53, 21)
    assert property_data_index == 177510

    # Test a specific cell (results from ResInsight UI)
    cell_index = 168143
    assert math.isclose(3627.17, cell_centers[cell_index].x, abs_tol=0.1)
    assert math.isclose(5209.75, cell_centers[cell_index].y, abs_tol=0.1)
    assert math.isclose(4179.6, cell_centers[cell_index].z, abs_tol=0.1)

    cell_corners = grid.cell_corners()
    assert len(cell_corners) == (dimensions.i * dimensions.j * dimensions.k)

    # Expected values from ResInsight UI
    expected_corners = [
        [3565.22, 5179.02, 4177.18],
        [3655.67, 5145.34, 4176.63],
        [3690.07, 5240.69, 4180.02],
        [3599.87, 5275.16, 4179.32],
        [3564.13, 5178.61, 4179.75],
        [3654.78, 5144.79, 4179.23],
        [3688.99, 5239.88, 4182.7],
        [3598.62, 5274.48, 4181.96],
    ]
    check_corner(cell_corners[cell_index].c0, expected_corners[0])
    check_corner(cell_corners[cell_index].c1, expected_corners[1])
    check_corner(cell_corners[cell_index].c2, expected_corners[2])
    check_corner(cell_corners[cell_index].c3, expected_corners[3])
    check_corner(cell_corners[cell_index].c4, expected_corners[4])
    check_corner(cell_corners[cell_index].c5, expected_corners[5])
    check_corner(cell_corners[cell_index].c6, expected_corners[6])
    check_corner(cell_corners[cell_index].c7, expected_corners[7])


def check_reek_grid_box(case: Case):
    assert len(case.grids()) == 1
    grid = case.grid(index=0)

    dimensions = grid.dimensions()
    assert dimensions.i == 21
    assert dimensions.j == 23
    assert dimensions.k == 14

    cell_centers = grid.cell_centers()
    total_size = dimensions.i * dimensions.j * dimensions.k
    assert len(cell_centers) == total_size

    poro = case.active_cell_property("INPUT_PROPERTY", "PORO", 0)
    assert len(poro) == total_size
    assert math.isclose(min(poro), 0.000928084715269506)
    assert math.isclose(max(poro), 0.351595014333725)


def test_load_roff_binary_grid(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/reek/reek_box_grid_w_props.roff"
    case = rips_instance.project.load_case(path=casePath)
    check_reek_grid_box(case)


def test_load_roff_ascii_grid(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/reek/reek_box_grid_w_props.roffasc"
    case = rips_instance.project.load_case(path=casePath)
    check_reek_grid_box(case)


def test_load_grdecl_grid(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/reek/reek_box_grid_w_props.grdecl"
    case = rips_instance.project.load_case(path=casePath)
    check_reek_grid_box(case)


def verify_load_grid_and_separate_properties(
    case: Case, property_name_and_paths: NameAndPath
):
    # Load case without properties
    available_properties = case.available_properties("INPUT_PROPERTY")
    for [name, _path] in property_name_and_paths.items():
        assert name not in available_properties

    grid = case.grid(index=0)
    dimensions = grid.dimensions()
    total_size = dimensions.i * dimensions.j * dimensions.k

    # Import properties to case
    case.import_properties(file_names=list(property_name_and_paths.values()))
    available_properties = case.available_properties("INPUT_PROPERTY")
    for [name, _path] in property_name_and_paths.items():
        assert name in available_properties
        property_values = case.active_cell_property("INPUT_PROPERTY", name, 0)
        assert len(property_values) == total_size


def test_load_grdecl_grid_with_separate_properties(rips_instance, initialize_test):
    grid_file_path: str = dataroot.PATH + "/reek/reek_box_grid_w_out_props.grdecl"
    property_name_and_paths: NameAndPath = {
        "EQLNUM": dataroot.PATH + "/reek/reek_box_EQLNUM_property.grdecl",
        "PORO": dataroot.PATH + "/reek/reek_box_PORO_property.grdecl",
    }
    case = rips_instance.project.load_case(path=grid_file_path)

    verify_load_grid_and_separate_properties(case, property_name_and_paths)


def test_load_roff_grid_with_separate_properties(rips_instance, initialize_test):
    grid_file_path: str = dataroot.PATH + "/reek/reek_box_grid_w_out_props.roffasc"
    property_name_and_paths: NameAndPath = {
        "EQLNUM": dataroot.PATH + "/reek/reek_box_EQLNUM_property.roffasc",
        "PORO": dataroot.PATH + "/reek/reek_box_PORO_property.roffasc",
    }
    case = rips_instance.project.load_case(path=grid_file_path)

    verify_load_grid_and_separate_properties(case, property_name_and_paths)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import App_pb2 as App__pb2
import Definitions_pb2 as Definitions__pb2
import PdmObject_pb2 as PdmObject__pb2


class AppStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetVersion = channel.unary_unary(
                '/rips.App/GetVersion',
                request_serializer=Definitions__pb2.Empty.SerializeToString,
                response_deserializer=App__pb2.Version.FromString,
                )
        self.Exit = channel.unary_unary(
                '/rips.App/Exit',
                request_serializer=Definitions__pb2.Empty.SerializeToString,
                response_deserializer=Definitions__pb2.Empty.FromString,
                )
        self.GetRuntimeInfo = channel.unary_unary(
                '/rips.App/GetRuntimeInfo',
                request_serializer=Definitions__pb2.Empty.SerializeToString,
                response_deserializer=App__pb2.RuntimeInfo.FromString,
                )
        self.GetPdmObject = channel.unary_unary(
                '/rips.App/GetPdmObject',
                request_serializer=Definitions__pb2.Empty.SerializeToString,
                response_deserializer=PdmObject__pb2.PdmObject.FromString,
                )


class AppServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetVersion(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Exit(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRuntimeInfo(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPdmObject(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AppServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetVersion': grpc.unary_unary_rpc_method_handler(
                    servicer.GetVersion,
                    request_deserializer=Definitions__pb2.Empty.FromString,
                    response_serializer=App__pb2.Version.SerializeToString,
            ),
            'Exit': grpc.unary_unary_rpc_method_handler(
                    servicer.Exit,
                    request_deserializer=Definitions__pb2.Empty.FromString,
                    response_serializer=Definitions__pb2.Empty.SerializeToString,
            ),
            'GetRuntimeInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRuntimeInfo,
                    request_deserializer=Definitions__pb2.Empty.FromString,
                    response_serializer=App__pb2.RuntimeInfo.SerializeToString,
            ),
            'GetPdmObject': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPdmObject,
                    request_deserializer=Definitions__pb2.Empty.FromString,
                    response_serializer=PdmObject__pb2.PdmObject.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'rips.App', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class App(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetVersion(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rips.App/GetVersion',
            Definitions__pb2.Empty.SerializeToString,
            App__pb2.Version.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Exit(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rips.App/Exit',
            Definitions__pb2.Empty.SerializeToString,
            Definitions__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRuntimeInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rips.App/GetRuntimeInfo',
            Definitions__pb2.Empty.SerializeToString,
            App__pb2.RuntimeInfo.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPdmObject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rips.App/GetPdmObject',
            Definitions__pb2.Empty.SerializeToString,
            PdmObject__pb2.PdmObject.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from rips.generated.generated_classes import (
    ModeledWellPath,
    StimPlanModel,
    WellPathGeometry,
    WellPathTarget,
)
import sys
import os

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips


def test_add_new_object_for_well_paths(rips_instance, initialize_test):
    well_path_coll = rips_instance.project.descendants(rips.WellPathCollection)[0]

    my_well_path = well_path_coll.add_new_object(rips.ModeledWellPath)
    my_well_path.name = "test"
    my_well_path.update()

    geometry = my_well_path.well_path_geometry()
    geometry.add_new_object(rips.WellPathTarget)
    geometry.add_new_object(rips.WellPathTarget)
    geometry.add_new_object(rips.WellPathTarget)
    assert len(geometry.well_path_targets()) == 3

    assert len(well_path_coll.well_paths()) == 1
    my_well_path_duplicate = well_path_coll.well_paths()[0]
    assert my_well_path_duplicate.name == "test"
    geometry_duplicate = my_well_path_duplicate.well_path_geometry()
    assert len(geometry_duplicate.well_path_targets()) == 3

    # Not allowed to add object of unrelated type
    invalid_object = geometry.add_new_object(rips.WellPath)
    assert invalid_object is None


def test_add_well_path_targets(rips_instance, initialize_test):
    well_path_coll = rips_instance.project.descendants(rips.WellPathCollection)[0]

    my_well_path = well_path_coll.add_new_object(rips.ModeledWellPath)
    my_well_path.name = "test"
    my_well_path.update()

    geometry = my_well_path.well_path_geometry()

    # Append well target with fixed azimuth
    coord = [2229.10, -833.74, -74.70]
    target = geometry.append_well_target(
        coord, use_fixed_azimuth=True, fixed_azimuth_value=110.1
    )
    assert target.use_fixed_inclination == False
    assert target.use_fixed_azimuth == True
    assert target.azimuth == 110.1
    assert target.inclination == 0.0

    # Append well target with fixed inclination
    coord = [4577.21, -3043.47, -87.15]
    target = geometry.append_well_target(
        coord, use_fixed_inclination=True, fixed_inclination_value=25.6
    )

    assert target.use_fixed_inclination == True
    assert target.use_fixed_azimuth == False
    assert target.azimuth == 0.0
    assert target.inclination == 25.6



from setuptools import setup, find_packages

with open('README.md') as f:
    readme = f.read()

with open('LICENSE') as f:
	license = f.read()

RIPS_DIST_VERSION = '2'
	
setup(
    name='rips',
    version='2023.06.0.' + RIPS_DIST_VERSION,
    description='Python Interface for ResInsight',
    long_description=readme,
    author='Ceetron Solutions',
    author_email='info@ceetronsolutions.com',
    url='http://www.resinsight.org',
    license=license,
    packages=['rips'],
    package_data={'rips': ['*.py', 'generated/*.py', 'PythonExamples/*.py', 'tests/*.py']},
    install_requires=['grpcio>=1.20.0', 'protobuf', 'wheel']
)



import sys
import os
import grpc
import pytest
import tempfile

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot


def test_10kAsync(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=casePath)

    resultChunks = case.active_cell_property_async("DYNAMIC_NATIVE", "SOIL", 1)
    mysum = 0.0
    count = 0
    for chunk in resultChunks:
        mysum += sum(chunk.values)
        count += len(chunk.values)
    average = mysum / count
    assert mysum == pytest.approx(621.768, abs=0.001)
    assert average != pytest.approx(0.0158893, abs=0.0000001)
    assert average == pytest.approx(0.0558893, abs=0.0000001)


def test_10kSync(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=casePath)

    results = case.active_cell_property("DYNAMIC_NATIVE", "SOIL", 1)
    mysum = sum(results)
    average = mysum / len(results)
    assert mysum == pytest.approx(621.768, abs=0.001)
    assert average != pytest.approx(0.0158893, abs=0.0000001)
    assert average == pytest.approx(0.0558893, abs=0.0000001)


def test_10k_set(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=casePath)

    results = case.active_cell_property("DYNAMIC_NATIVE", "SOIL", 1)
    case.set_active_cell_property(results, "GENERATED", "SOIL", 1)


def test_10k_set_out_of_bounds(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=casePath)

    results = case.active_cell_property("DYNAMIC_NATIVE", "SOIL", 1)
    results.append(5.0)
    with pytest.raises(grpc.RpcError):
        assert case.set_active_cell_property(results, "GENERATED", "SOIL", 1)


def test_10k_set_out_of_bounds_client(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=casePath)

    results = case.active_cell_property("DYNAMIC_NATIVE", "SOIL", 1)
    case.chunk_size = len(results)
    results.append(5.0)
    with pytest.raises(IndexError):
        assert case.set_active_cell_property(results, "GENERATED", "SOIL", 1)


def createResult(poroChunks, permxChunks):
    for poroChunk, permxChunk in zip(poroChunks, permxChunks):
        resultChunk = []
        for poro, permx in zip(poroChunk.values, permxChunk.values):
            resultChunk.append(poro * permx)
        yield resultChunk


def checkResults(poroValues, permxValues, poropermxValues):
    for poro, permx, poropermx in zip(poroValues, permxValues, poropermxValues):
        recalc = poro * permx
        assert recalc == pytest.approx(poropermx, rel=1.0e-10)


def test_10k_PoroPermX(rips_instance, initialize_test):
    casePath = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=casePath)

    poroChunks = case.active_cell_property_async("STATIC_NATIVE", "PORO", 0)
    permxChunks = case.active_cell_property_async("STATIC_NATIVE", "PERMX", 0)

    case.set_active_cell_property_async(
        createResult(poroChunks, permxChunks), "GENERATED", "POROPERMXAS", 0
    )

    poro = case.active_cell_property("STATIC_NATIVE", "PORO", 0)
    permx = case.active_cell_property("STATIC_NATIVE", "PERMX", 0)
    poroPermX = case.active_cell_property("GENERATED", "POROPERMXAS", 0)

    checkResults(poro, permx, poroPermX)


def test_exportPropertyInView(rips_instance, initialize_test):
    case_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(case_path)
    case.create_view()
    with tempfile.TemporaryDirectory(prefix="rips") as tmpdirname:
        print("Temporary folder: ", tmpdirname)
        rips_instance.set_export_folder(export_type="PROPERTIES", path=tmpdirname)
        case = rips_instance.project.cases()[0]
        view = case.views()[0]
        view.export_property()
        expected_file_name = case.name + "-" + str("3D_View") + "-" + "T0" + "-SOIL"
        full_path = tmpdirname + "/" + expected_file_name
        assert os.path.exists(full_path)



#
#  Copyright (C) 2019- Equinor ASA
# 
#  ResInsight is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  ResInsight is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or
#  FITNESS FOR A PARTICULAR PURPOSE.
# 
#  See the GNU General Public License at <http://www.gnu.org/licenses/gpl.html> 
#  for more details.
#
#  Python version of RiaVersionInfo.h
#  Just sets version constants

RESINSIGHT_MAJOR_VERSION = "2023"
RESINSIGHT_MINOR_VERSION = "06"
RESINSIGHT_PATCH_VERSION = "0"



# Load ResInsight Processing Server Client Library
import rips
import tempfile
from os.path import expanduser
from pathlib import Path

# Connect to ResInsight instance
resinsight = rips.Instance.find()


home_dir = expanduser("~")


properties = [
    ("STATIC_NATIVE", "INDEX_K", 0),
    ("STATIC_NATIVE", "PORO", 0),
    ("STATIC_NATIVE", "PERMX", 0),
    ("DYNAMIC_NATIVE", "PRESSURE", 0),
]

export_folder = tempfile.mkdtemp()

directory_path = "resprojects/webviz-subsurface-testdata/reek_history_match/"


case_file_paths = []
num_realizations = 9
num_iterations = 4


for realization in range(0, num_realizations):
    for iteration in range(0, num_iterations):
        realization_dir = "realization-" + str(realization)
        iteration_dir = "iter-" + str(iteration)
        egrid_name = "eclipse/model/5_R001_REEK-" + str(realization) + ".EGRID"
        path = Path(
            home_dir, directory_path, realization_dir, iteration_dir, egrid_name
        )
        case_file_paths.append(path)

for path in case_file_paths:
    # Load a case
    path_name = path.as_posix()
    grid_only = True
    case = resinsight.project.load_case(path_name, grid_only)

    # Load some wells
    well_paths = resinsight.project.import_well_paths(
        well_path_files=[
            Path(home_dir, directory_path, "wellpaths", "Well-1.dev").as_posix(),
            Path(home_dir, directory_path, "wellpaths", "Well-2.dev").as_posix(),
        ]
    )

    if resinsight.project.has_warnings():
        for warning in resinsight.project.warnings():
            print(warning)

    well_log_plot_collection = resinsight.project.descendants(
        rips.WellLogPlotCollection
    )[0]

    for well_path in well_paths:
        print(
            "Generating las file for well: " + well_path.name + " in case: " + path_name
        )

        well_log_plot = well_log_plot_collection.new_well_log_plot(case, well_path)

        # Create a track for each property
        for prop_type, prop_name, time_step in properties:
            track = well_log_plot.new_well_log_track(
                "Track: " + prop_name, case, well_path
            )

            c = track.add_extraction_curve(
                case, well_path, prop_type, prop_name, time_step
            )

        parent_path = path.parent
        export_folder_path = Path(parent_path, "lasexport")
        export_folder_path.mkdir(parents=True, exist_ok=True)

        export_folder = export_folder_path.as_posix()
        well_log_plot.export_data_as_las(export_folder=export_folder)

    resinsight.project.close()



######################################################################################
# This example prints information about the grids of all cases in the current project
######################################################################################

import rips

resinsight = rips.Instance.find()

cases = resinsight.project.cases()
print("Number of cases found: ", len(cases))
for case in cases:
    print(case.name)
    grids = case.grids()
    print("Number of grids: ", len(grids))
    for grid in grids:
        print("Grid dimensions: ", grid.dimensions())



# The path here is intended to be used when pytest is launced from the source tree of the ResInsight repository
# This enables use of test datasets from the TestModels folder
PATH = "../../../TestModels"



###########################################################################################
# This example will synchronously calculate the average value for SOIL for all time steps
###########################################################################################
import rips
import itertools
import time

resinsight = rips.Instance.find()

start = time.time()

# Get the case with case id 0
case = resinsight.project.case(case_id=0)

# Get a list of all time steps
time_steps = case.time_steps()

averages = []
for i in range(0, len(time_steps)):
    # Get a list of all the results for time step i
    results = case.active_cell_property("DYNAMIC_NATIVE", "SOIL", i)
    mysum = sum(results)
    averages.append(mysum / len(results))

end = time.time()
print("Time elapsed: ", end - start)
print(averages)



###################################################################
# This example demonstrates the use of ResInsight exceptions
# for proper error handling
###################################################################

import rips
import grpc
import tempfile

resinsight = rips.Instance.find()

case = None

# Try loading a non-existing case. We should get a grpc.RpcError exception from the server
try:
    case = resinsight.project.load_case("Nonsense")
except grpc.RpcError as e:
    print(
        "Expected Server Exception Received while loading case: ", e.code(), e.details()
    )

# Try loading well paths from a non-existing folder.  We should get a grpc.RpcError exception from the server
try:
    well_path_files = resinsight.project.import_well_paths(
        well_path_folder="NONSENSE/NONSENSE"
    )
except grpc.RpcError as e:
    print(
        "Expected Server Exception Received while loading wellpaths: ",
        e.code(),
        e.details(),
    )

# Try loading well paths from an existing but empty folder. We should get a warning.
try:
    with tempfile.TemporaryDirectory() as tmpdirname:
        well_path_files = resinsight.project.import_well_paths(
            well_path_folder=tmpdirname
        )
        assert len(well_path_files) == 0
        assert resinsight.project.has_warnings()
        print("Should get warnings below")
        for warning in resinsight.project.warnings():
            print(warning)
except grpc.RpcError as e:
    print("Unexpected Server Exception caught!!!", e)

case = resinsight.project.case(case_id=0)
if case is not None:
    results = case.active_cell_property("STATIC_NATIVE", "PORO", 0)
    active_cell_count = len(results)

    # Send the results back to ResInsight inside try / except construct
    try:
        case.set_active_cell_property(results, "GENERATED", "POROAPPENDED", 0)
        print("Everything went well as expected")
    except:  # Match any exception, but it should not happen
        print("Ooops!")

    # Add another value, so this is outside the bounds of the active cell result storage
    results.append(1.0)

    # This time we should get a grpc.RpcError exception, which is a server side error.
    try:
        case.set_active_cell_property(results, "GENERATED", "POROAPPENDED", 0)
        print("Everything went well??")
    except grpc.RpcError as e:
        print("Expected Server Exception Received: ", e)
    except IndexError:
        print("Got index out of bounds error. This shouldn't happen here")

    # With a chunk size exactly matching the active cell count the server will not
    # be able to see any error as it will successfully close the stream after receiving
    # the correct number of values, even if the python client has more chunks to send
    case.chunk_size = active_cell_count

    try:
        case.set_active_cell_property(results, "GENERATED", "POROAPPENDED", 0)
        print("Everything went well??")
    except grpc.RpcError as e:
        print("Got unexpected server exception", e, "This should not happen now")
    except IndexError:
        print("Got expected index out of bounds error on client side")



############################################################################
# This script will export snapshots for two properties in every loaded case
# And put them in a snapshots folder in the same folder as the case grid
############################################################################
import os
import rips

# Load instance
resinsight = rips.Instance.find()
cases = resinsight.project.cases()

# Set main window size
resinsight.set_main_window_size(width=800, height=500)

n = 5  # every n-th time_step for snapshot
property_list = ["SOIL", "PRESSURE"]  # list of parameter for snapshot

print("Looping through cases")
for case in cases:
    print("Case name: ", case.name)
    print("Case id: ", case.id)
    # Get grid path and its folder name
    case_path = case.file_path
    folder_name = os.path.dirname(case_path)

    # create a folder to hold the snapshots
    dirname = os.path.join(folder_name, "snapshots")

    if os.path.exists(dirname) is False:
        os.mkdir(dirname)

    print("Exporting to folder: " + dirname)
    resinsight.set_export_folder(export_type="SNAPSHOTS", path=dirname)

    time_steps = case.time_steps()
    print("Number of time_steps: " + str(len(time_steps)))

    for view in case.views():
        for property in property_list:
            view.apply_cell_result(
                result_type="DYNAMIC_NATIVE", result_variable=property
            )
        for time_step in range(0, len(time_steps), 10):
            view.set_time_step(time_step=time_step)
            view.export_snapshot()



#######################################
# This example connects to ResInsight
#######################################
import rips

resinsight = rips.Instance.find()

if resinsight is None:
    print("ERROR: could not find ResInsight")
else:
    print("Successfully connected to ResInsight")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: SimulationWell.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Definitions_pb2 as Definitions__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14SimulationWell.proto\x12\x04rips\x1a\x11\x44\x65\x66initions.proto\"M\n\x15SimulationWellRequest\x12\x0f\n\x07\x63\x61se_id\x18\x01 \x01(\x05\x12\x11\n\twell_name\x18\x02 \x01(\t\x12\x10\n\x08timestep\x18\x03 \x01(\x05\":\n\x14SimulationWellStatus\x12\x11\n\twell_type\x18\x01 \x01(\t\x12\x0f\n\x07is_open\x18\x02 \x01(\x08\"~\n\x16SimulationWellCellInfo\x12\x18\n\x03ijk\x18\x01 \x01(\x0b\x32\x0b.rips.Vec3i\x12\x12\n\ngrid_index\x18\x02 \x01(\x05\x12\x0f\n\x07is_open\x18\x03 \x01(\x08\x12\x11\n\tbranch_id\x18\x04 \x01(\x05\x12\x12\n\nsegment_id\x18\x05 \x01(\x05\"I\n\x1bSimulationWellCellInfoArray\x12*\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x1c.rips.SimulationWellCellInfo2\xc2\x01\n\x0eSimulationWell\x12T\n\x17GetSimulationWellStatus\x12\x1b.rips.SimulationWellRequest\x1a\x1a.rips.SimulationWellStatus\"\x00\x12Z\n\x16GetSimulationWellCells\x12\x1b.rips.SimulationWellRequest\x1a!.rips.SimulationWellCellInfoArray\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'SimulationWell_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _globals['_SIMULATIONWELLREQUEST']._serialized_start=49
  _globals['_SIMULATIONWELLREQUEST']._serialized_end=126
  _globals['_SIMULATIONWELLSTATUS']._serialized_start=128
  _globals['_SIMULATIONWELLSTATUS']._serialized_end=186
  _globals['_SIMULATIONWELLCELLINFO']._serialized_start=188
  _globals['_SIMULATIONWELLCELLINFO']._serialized_end=314
  _globals['_SIMULATIONWELLCELLINFOARRAY']._serialized_start=316
  _globals['_SIMULATIONWELLCELLINFOARRAY']._serialized_end=389
  _globals['_SIMULATIONWELL']._serialized_start=392
  _globals['_SIMULATIONWELL']._serialized_end=586
# @@protoc_insertion_point(module_scope)



##############################################################################
# This example will create a derived result for each time step asynchronously
##############################################################################

import rips
import time


# Internal function for creating a result from a small chunk of soil and porv results
# The return value of the function is a generator for the results rather than the result itself.
def create_result(soil_chunks, porv_chunks):
    for soil_chunk, porv_chunk in zip(soil_chunks, porv_chunks):
        resultChunk = []
        number = 0
        for soil_value, porv_value in zip(soil_chunk.values, porv_chunk.values):
            resultChunk.append(soil_value * porv_value)
        # Return a Python generator
        yield resultChunk


resinsight = rips.Instance.find()
start = time.time()
case = resinsight.project.cases()[0]
timeStepInfo = case.time_steps()

# Get a generator for the porv results. The generator will provide a chunk each time it is iterated
porv_chunks = case.active_cell_property_async("STATIC_NATIVE", "PORV", 0)

# Read the static result into an array, so we don't have to transfer it for each iteration
# Note we use the async method even if we synchronise here, because we need the values chunked
# ... to match the soil chunks
porv_array = []
for porv_chunk in porv_chunks:
    porv_array.append(porv_chunk)

for i in range(0, len(timeStepInfo)):
    # Get a generator object for the SOIL property for time step i
    soil_chunks = case.active_cell_property_async("DYNAMIC_NATIVE", "SOIL", i)
    # Create the generator object for the SOIL * PORV derived result
    result_generator = create_result(soil_chunks, iter(porv_array))
    # Send back the result asynchronously with a generator object
    case.set_active_cell_property_async(
        result_generator, "GENERATED", "SOILPORVAsync", i
    )

end = time.time()
print("Time elapsed: ", end - start)

print("Transferred all results back")

view = case.views()[0].apply_cell_result("GENERATED", "SOILPORVAsync")



# Access to environment variables and path tools
import os

# Load ResInsight Processing Server Client Library
import rips

# Connect to ResInsight instance
resinsight = rips.Instance.find()

# This requires the TestModels to be installed with ResInsight (RESINSIGHT_BUNDLE_TESTMODELS):
resinsight_exe_path = os.environ.get("RESINSIGHT_EXECUTABLE")

# Get the TestModels path from the executable path
resinsight_install_path = os.path.dirname(resinsight_exe_path)
test_models_path = os.path.join(resinsight_install_path, "TestModels")
path_name = os.path.join(
    test_models_path, "TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
)
case = resinsight.project.load_case(path_name)

# Save the project to file
home_dir = os.path.expanduser("~")
project_path = home_dir + "/new-project.rsp"
print("Saving project to: ", project_path)
resinsight.project.save(project_path)
